module.exports = function (grunt) {

	var backendModules = [
	    '../backend'
	];
	
	var directories = ['general', 'utilities', 'repositories', 'services', 'controllers'];
	
	var directoriesCRMTemplates = ['directives', 'controllers'];
	
	var dependenciesPath = [
		"bower_components/zhuaclfrontend/assets/dist/main.js",
		'bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js',
		'bower_components/zh-preferences/dist/preferences.js',
		"bower_components/zh-filter-preferences/dist/filter-preferences.min.js",
		"bower_components/zh-layout-preferences/dist/layout-preferences.min.js",
        'bower_components/zh-view-preferences/dist/view-preferences.min.js',
		"bower_components/zh-interval-component/dist/zh-interval-component.min.js",
		"bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js",
		"bower_components/zh-grid-configuration/dist/zh-grid-configuration.js",
		"bower_components/zhulog/dist/main.min.js",
		"bower_components/zh-scheduler/dist/zh-scheduler.min.js",
		"bower_components/zh-rich-text/dist/zh-rich-text.min.js",
		"bower_components/zh-image-uploader/dist/zh-image-uploader.min.js",
		"bower_components/zh-reports/dist/zhReports.js",
		"bower_components/devtools/dist/developer-enrichment.min.js",
		"bower_components/zh-cache-bypass/src/CacheBypass.js",
		"bower_components/zh-charts/dist/zh-charts.js",
		"bower_components/tek-libraries/dist/main.js",
		"bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js"
	];
	
	var cssDependenciesPath = [
		"bower_components/zh-rich-text/dist/zh-rich-text.css",
		"bower_components/zh-charts/dist/zh-charts.css",
		"bower_components/zh-preferences/dist/style/style.css",
		"bower_components/zh-preferences/dist/style/colors/styles-inflightor.css",
		"bower_components/zh-grid-navigation/dist/style.css",
		"bower_components/zh-grid-configuration/dist/zh-grid-configuration.css",
		"bower_components/zh-scheduler/dist/zh-scheduler.css",
		"bower_components/zh-image-uploader/dist/zh-image-uploader.css",
		"crm_templates/css/kanbanTicket.css",
		"crm_templates/css/text-with-native.css",
		"bower_components/zh-reports/dist/style/colors/styles-inflightor.css"
	];
	
	grunt.initConfig({
		concat: {
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSPaths(),
				dest: './assets/js/app.js'
			},
			css: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: getCSSPaths(),
				dest: './assets/css/app.css'
			},
			vendor: {
				options: {
					process: function (src, filepath) {
						return '\n' + '// FILE: ' + filepath + '\n' + src;
					}
				},
				src: [dependenciesPath],
				dest: './assets/js/vendor.js'
			},
			vendorCss: {
				options: {
					process: function (src, filepath) {
						return '\n' + '/* FILE: ' + filepath + '*/\n' + src;
					}
				},
				src: [cssDependenciesPath],
				dest: './assets/css/vendor.css'
			},
			jsCRMTemplates: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSPathsForCRMTemplates(),
				dest: './assets/js/appCustom.js'
			},
			cssCRMTemplates: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: getCSSPathsForCRMTemplates(),
				dest: './assets/css/appCustom.css'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'./assets/js/app.min.js': getJSPaths()
				}
			}
		},
		jshint: {
			all: getJSPathsForJSHINT().concat(["GruntFile.js"]),
			options: {
				evil: true
			}
		},
		watch: {
			js: {
				files: getJSPaths().concat(["GruntFile.js"]),
				tasks: [ /*'jshint',*/ 'concat:js']
			},
			css: {
				files: getCSSPaths(),
				tasks: ['concat:css']
			}
		},
		cssmin: {
			dist: {
				src: getCSSPaths(),
				dest: './assets/css/app.min.css'
			}
		},
		zhIdGenerator: {
		    options: {
		        jsonPath: ["./assets/json"],
		        menuPath: ["./assets/json/menu.json"],
		        replaceIds: false
		    }
		},
		buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        buildEnvironment: {
		    development: {
		        configFile: "config/devEnvironment.json"
		    },
		    dev: {
		        configFile: "config/devEnvironment.json"
		    },
            test: {
                configFile: "config/testEnvironment.json"
            },
            deploy: {
                configFile: "config/prodEnvironment.json"
            },
		    "srcFolders": ["./assets/js", "./config", "./assets/json", "./", "../backend/config", "../backend/src/Teknisa/CRM/Job"]
		}
	});
	
	function getJSPathsForCRMTemplates() {
		var paths = [];
		directoriesCRMTemplates.forEach(function (directory) {
			paths.push('./crm_templates/' + directory + '/**/*.js');
		});
		return paths;
	}
	
	function getCSSPathsForCRMTemplates() {
		var paths = [];
		paths.push('./crm_templates/css/**/*.css');
		return paths;
	}
	
	function getJSPaths() {
		var paths = [];
		directories.forEach(function (directory) {
			paths.push('./assets/js/' + directory + '/**/*.js');
		});
		return paths;
	}
	
	function getJSPathsForJSHINT() {
		var paths = [];
		directories.forEach(function (directory) {
			if(directory!=='apis')
				paths.push('./assets/js/' + directory + '/**/*.js');
		});
		return paths;
	}

	function getCSSPaths() {
		var paths = [];
		paths.push('./assets/css/**/*.css');
		paths.push('!./assets/css/app.css');
		paths.push('!./assets/css/app.min.css');
		return paths;
	}

    function inArray(array, value) {
        return array.filter(function(position) {
            return position == value;
        }).length;
    }
	
	function buildFileContent(srcFiles, filePath, directory) {
        var parts = filePath.split('/');
        parts.shift();
        parts.shift();
        var filename = parts[parts.length - 1];
        var extension = filename.split('.').pop().toUpperCase();
        var files = ['routes', 'factories', 'services', 'controllers'];
        if ((extension == 'JSON' || extension == 'XML')) {
            if (directory == 'datasources') {
                filename = parts.pop();
                filename = filename.replace(/\.[^/.]+$/, "");
                if (filePath.search('backend') != -1) {
                	srcFiles[directory][filename] = './../' + parts.join('/');
                } else {
                	srcFiles[directory][filename] = './../../modules/' + parts.join('/');
                }
            } else {
                if (inArray(files, filename.split('.')[0])) {
                    var isModule = (parts[0] != 'config')? '/../modules/' : '/';
                    srcFiles[directory].push(isModule + parts.join('/'));
                }
            }
        }
    }

	grunt.registerTask('backendStep', 'build backend config files', function () {
		var srcFiles = {
			config: [],
			datasources: {}
		};
		var directories = ['config', 'datasources'];
		var dest = {
			config: '../backend/config/servicesMapping.json',
			datasources: '../backend/config/dataSourcesMapping.json'
		};
		directories.forEach(function (directory) {
			backendModules.forEach(function (module) {

				var moduleParts = module.split('/');
				var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
				var subModules = grunt.file.expand({
					filter: 'isDirectory'
				}, module + '/src/Teknisa/' + moduleName + '/Modules/*');

				subModules.reverse();

				subModules.forEach(function (subModule) {
					grunt.file.expand({
						filter: 'isFile'
					}, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
						buildFileContent(srcFiles, filePath, directory);
					});
				});

				grunt.file.expand({
					filter: 'isFile'
				}, module + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
			});
		});
        
		grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
		
	});
	
	    grunt.registerTask("zhIdRemovator", "Remove ids for json items like widgets and actions", function(arg1) {
        var jsonPath = "../mobile/assets/json/containers";
        var fileName = arg1;
        if (typeof fileName == "undefined") {
            grunt.log.error("Invalid Argument. The file name must be informed.");
        }
        var filePath = jsonPath + "/" + fileName;
        var fileObject = grunt.file.readJSON(filePath);
        var countId = 0;

        function isObject(value) {
            if (Object.prototype.toString.call(value) === '[object Object]' || Object.prototype.toString.call(value) === '[object Array]') {
                return true
            }
        }

        function removeIdFromObject(object) {
            if (object['id']) {
                grunt.log.writeln("remove Id from: " + object['name']);
                delete object.id;
                countId++;
            }
        }

        function objectPropertiesIterator(object) {
            for (var property in object) {
                if (isObject(object[property])) {
                    removeIdFromObject(object[property]);
                    objectPropertiesIterator(object[property]);
                } else if (Array.isArray(object[property])) {
                    arrayIterator(object[property]);
                }
            }
        }

        function arrayIterator(array) {
            array.forEach(function(item) {
                if (isObject(item)) {
                    removeIdFromObject(item);
                    objectPropertiesIterator(item);
                } else if (Array.isArray(item)) {
                    arrayIterator(item);
                }
            });
        }

        if (isObject(fileObject)) {
            removeIdFromObject(fileObject);
            objectPropertiesIterator(fileObject);
            grunt.file.write(filePath, JSON.stringify(fileObject, null, 4));
            grunt.log.writeln(countId + " id's removed from " + fileName);
            grunt.log.writeln("File rewrited");
        } else {
            grunt.log.error("Invalid File. The file could not be found.");
        }
    });


	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-cssmin');
	grunt.loadNpmTasks('grunt-zh-id-generator');
	grunt.loadNpmTasks('grunt-zhuacl-structure');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	
	grunt.registerTask('default', ['backendStep', 'concat', 'buildAclStruct']);
	grunt.registerTask('dist', ['backendStep', 'uglify', 'cssmin']);
	
	grunt.registerTask('init_project', [
        'backendStep',
        'concat',
        'jshint'
    ]);
};
