/*global Util, $, Configuration*/
function WizardProspectController(ScreenService, templateManager, restEngine, metaDataFactory) {

    this.executeQuery = function(argsRow, mode) {

        if(Util.isEmptyOrBlank(argsRow.DSQUERY)){
            ScreenService.showMessage('Favor informar a query desejada.', 'ALERT');
        } else {
            restEngine.post('/executeQuery', {
                'requestType':'Row', 
                'row': {
                    query: argsRow.DSQUERY.toUpperCase(),
                    type:  (mode === 'fetch') ? 'fecth' : 'execute'
                }
            }).then(function (data) {
                try {
                    if (!Util.isUndefined(data.messages)) {
                        ScreenService.showMessage(data.messages[0].message, 'ERROR');
                    } else {
                        var container = templateManager.container;
                        var widgetResult = container.getWidget('wizardProspectGrid');

                        widgetResult.fields = [];

                        if (data.dataset.resultQuery.length > 0 && (data.dataset.resultQuery[0] !== null && data.dataset.resultQuery[0] !== true)) {
                            var resultQueryColumns = data.dataset.resultQuery[0];
                            $.each(resultQueryColumns, function (key, value) {
                                var obj = {
                                    "name": key,
                                    "label": key,
                                    "isVisible": true,
                                    "showOnList": true,
                                    "isReadOnly": true,
                                    "template": "field/text.html",
                                    "events": []
                                };

                                metaDataFactory.factoryField(obj, widgetResult, true);
                            });

                            widgetResult.dataSource.data = data.dataset.resultQuery;

                            templateManager.updateTemplate();
                            widgetResult.redraw();
                            $(window).resize();

                            ScreenService.closePopup();
                            ScreenService.successNotification("Query executada com sucesso");
                            widgetResult.activate();
                        } else {
                            if(mode === 'fetch')
                                ScreenService.showMessage('Dados não encontrados na consulta.');
                            else if(data.dataset.resultQuery[0] === true){
                                ScreenService.closePopup();
                                ScreenService.successNotification("Query executada com sucesso");
                                widgetResult.activate();
                            }else if(data.dataset.resultQuery[0] === null){
                                ScreenService.successNotification("Falha na execução da query");
                            }
                        }
                    }
                } catch (e) {
                    ScreenService.showMessage(e, 'ERROR');
                }

            }).catch(function(error){
                var message = error.data.error;
                message = message.slice(message.indexOf('ORA'), message.length);
                ScreenService.showMessage(message);        
            });
        }

    };

    this.openPopupQuery = function() {

        var container = templateManager.container;
        var widget = container.getWidget('wizardProspectPopup');

        ScreenService.openPopup(widget);
    };
    
    this.checkQuery = function(row){
        var container = templateManager.container;
        var widget = container.getWidget('wizardProspectPopup');
        
        var query = row['DSQUERY'];
        var actionQuery = widget.getAction('QUERY');
        var actionExecute = widget.getAction('EXECUTE');
        
        if(query){
            query = query.toUpperCase();
            if(query.indexOf('SELECT') !== -1 || query.indexOf('DESC') !== -1){
                actionQuery.isVisible = true;
                actionExecute.isVisible = false;
            }
            
            if(query.indexOf('INSERT') !== -1 || query.indexOf('UPDATE') !== -1 || query.indexOf('DELETE') !== -1 || query.indexOf('ALTER') !== -1 || query.indexOf('BEGIN') !== -1 || query.indexOf('CREATE') !== -1 || query.indexOf('DECLARE') !== -1){
                actionQuery.isVisible = false;
                actionExecute.isVisible = true;
            }
            widget.activate();
        }
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('WizardProspectController', WizardProspectController);
});